package org.openiec61850.scsm.mms.client;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.concurrent.BlockingQueue;

import org.openiec61850.jmms.iso.acse.AcseAssociation;
import org.openiec61850.jmms.mms.asn1.ConfirmedRequestPdu;
import org.openiec61850.jmms.mms.asn1.MmsPdu;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class MmsScsmClientReceiver extends Thread {

	private static Logger logger = LoggerFactory.getLogger(MmsScsmClientReceiver.class);

	private AcseAssociation acseAssociation;

	private BlockingQueue<MmsPdu> incomingResponses;
	private BlockingQueue<MmsPdu> incomingReports;

	private boolean queueReports = false;
	private boolean stopped = false;

	public MmsScsmClientReceiver(AcseAssociation acseAssociation, BlockingQueue<MmsPdu> incomingResponses,
			BlockingQueue<MmsPdu> incomingReports) {
		this.acseAssociation = acseAssociation;
		this.incomingResponses = incomingResponses;
		this.incomingReports = incomingReports;
	}

	public void run() {
		try {
			ByteBuffer encodedResponsePdu;
			while (true) {
				encodedResponsePdu = acseAssociation.receive();

				MmsPdu decodedResponsePdu = new MmsPdu();
				try {
					decodedResponsePdu.decode(
							new ByteArrayInputStream(encodedResponsePdu.array(), encodedResponsePdu.arrayOffset()
									+ encodedResponsePdu.position(), encodedResponsePdu.limit()
									- encodedResponsePdu.position()), null);
				} catch (IOException e) {
					logger.error("unable to decode received message: " + e.getMessage());
					acseAssociation.disconnect();
					try {
						incomingResponses.put(new MmsPdu(new ConfirmedRequestPdu(), null, null, null, null, null));
						incomingReports.put(new MmsPdu(new ConfirmedRequestPdu(), null, null, null, null, null));
					} catch (InterruptedException e1) {
					}

				}

				if (decodedResponsePdu.unconfirmedPDU != null) {
					if (queueReports != false) {
						try {
							incomingReports.put(decodedResponsePdu);
						} catch (InterruptedException e) {
						}
					}
					else {
						logger.debug("discarding report because reports are disabled");
					}
				}
				else {
					try {
						incomingResponses.put(decodedResponsePdu);
					} catch (InterruptedException e) {
					}
				}

			}
		} catch (IOException e) {
			if (stopped == false) {
				acseAssociation.disconnect();
				try {
					logger.info("server closed connection unexpectedly");
					incomingResponses.put(new MmsPdu(new ConfirmedRequestPdu(), null, null, null, null, null));
					incomingReports.put(new MmsPdu(new ConfirmedRequestPdu(), null, null, null, null, null));
				} catch (InterruptedException e1) {
				}
			}
		}
	}

	public void enableReportQueueing() {
		queueReports = true;
	}

	public void disableReportQueueing() {
		queueReports = false;
	}

	public void close() {
		stopped = true;
		acseAssociation.close();
	}

	public void disconnect() {
		stopped = true;
		acseAssociation.disconnect();
	}

}
